@@ -108,6 +108,7 @@ static char *s_chtml10_end_menu_tag       (void *pdoc, Node *node);
 static char *s_chtml10_start_plaintext_tag(void *pdoc, Node *node);
 static char *s_chtml10_start_plaintext_tag_inner(void  *pdoc, Node *node);
 static char *s_chtml10_end_plaintext_tag  (void *pdoc, Node *node);
+static char *s_chtml10_newline_mark       (void *pdoc, Node *node);
 
 static void  s_init_chtml10(chtml10_t *chtml, Doc *doc, request_rec *r, device_table *spec);
 
@@ -381,6 +382,11 @@ tag_handler chtml10_handler[] = {
     NULL,
     NULL,
   },
+  /* tagNLMARK */
+  {
+    s_chtml10_newline_mark,
+    NULL,
+  },
 };
 
 
@@ -590,8 +596,6 @@ s_chtml10_start_html_tag(void *pdoc, Node *UNUSED(node))
   /* start HTML tag                                                           */
   /*--------------------------------------------------------------------------*/
   W_L("<html>");
-  W_NLCODE();
-
   return chtml10->out;
 }
 
@@ -617,8 +621,6 @@ s_chtml10_end_html_tag(void *pdoc, Node *UNUSED(child))
   r       = doc->r;
 
   W_L("</html>");
-  W_NLCODE();
-
   return chtml10->out;
 }
 
@@ -679,7 +681,6 @@ s_chtml10_start_head_tag(void *pdoc, Node *UNUSED(node))
   r       = doc->r;
 
   W_L("<head>");
-  W_NLCODE();
 
   return chtml10->out;
 }
@@ -705,7 +706,6 @@ s_chtml10_end_head_tag(void *pdoc, Node *UNUSED(child))
   r       = doc->r;
 
   W_L("</head>");
-  W_NLCODE();
 
   return chtml10->out;
 }
@@ -731,7 +731,6 @@ s_chtml10_start_ol_tag(void *pdoc, Node *UNUSED(node))
   r       = doc->r;
 
   W_L("<ol>");
-  W_NLCODE();
 
   return chtml10->out;
 }
@@ -757,7 +756,6 @@ s_chtml10_end_ol_tag(void *pdoc, Node *UNUSED(child))
   r       = doc->r;
 
   W_L("</ol>");
-  W_NLCODE();
 
   return chtml10->out;
 }
@@ -783,7 +781,6 @@ s_chtml10_start_ul_tag(void *pdoc, Node *UNUSED(node))
   r          = doc->r;
 
   W_L("<ul>");
-  W_NLCODE();
 
   return chtml10->out;
 }
@@ -809,7 +806,6 @@ s_chtml10_end_ul_tag(void *pdoc, Node *UNUSED(child))
   r       = doc->r;
 
   W_L("</ul>");
-  W_NLCODE();
 
   return chtml10->out;
 }
@@ -917,7 +913,6 @@ s_chtml10_end_h1_tag(void *pdoc, Node *UNUSED(child))
   r       = doc->r;
 
   W_L("</h1>");
-  W_NLCODE();
 
   return chtml10->out;
 }
@@ -982,7 +977,6 @@ s_chtml10_end_h2_tag(void *pdoc, Node *UNUSED(child))
   r       = doc->r;
 
   W_L("</h2>");
-  W_NLCODE();
 
   return chtml10->out;
 }
@@ -1049,7 +1043,6 @@ s_chtml10_end_h3_tag(void *pdoc, Node *UNUSED(child))
   r       = doc->r;
 
   W_L("</h3>");
-  W_NLCODE();
 
   return chtml10->out;
 }
@@ -1112,7 +1105,6 @@ s_chtml10_end_h4_tag(void *pdoc, Node *UNUSED(child))
   doc      = chtml10->doc;
 
   W_L("</h4>");
-  W_NLCODE();
 
   return chtml10->out;
 }
@@ -1177,7 +1169,6 @@ s_chtml10_end_h5_tag(void *pdoc, Node *UNUSED(child))
   doc     = chtml10->doc;
 
   W_L("</h5>");
-  W_NLCODE();
 
   return chtml10->out;
 }
@@ -1240,7 +1231,6 @@ s_chtml10_end_h6_tag(void *pdoc, Node *UNUSED(child))
   doc     = chtml10->doc;
 
   W_L("</h6>");
-  W_NLCODE();
 
   return chtml10->out;
 }
@@ -1287,7 +1277,6 @@ s_chtml10_end_title_tag(void *pdoc, Node *UNUSED(child))
   doc     = chtml10->doc;
 
   W_L("</title>");
-  W_NLCODE();
 
   return chtml10->out;
 }
@@ -1328,7 +1317,6 @@ s_chtml10_start_base_tag(void *pdoc, Node *node)
   }
 
   W_L(">");
-  W_NLCODE();
 
   return chtml10->out;
 }
@@ -1434,7 +1422,6 @@ s_chtml10_start_body_tag(void *pdoc, Node *node)
   }
 
   W_L(">");
-  W_NLCODE();
 
   return chtml10->out;
 }
@@ -1458,7 +1445,6 @@ s_chtml10_end_body_tag(void *pdoc, Node *UNUSED(child))
   doc     = chtml10->doc;
 
   W_L("</body>");
-  W_NLCODE();
 
   return chtml10->out;
 }
@@ -1651,7 +1637,7 @@ s_chtml10_end_a_tag(void *pdoc, Node *UNUSED(child))
   r       = doc->r;
 
   W_L("</a>");
-  W_NLCODE();
+
   return chtml10->out;
 }
 
@@ -1695,7 +1681,7 @@ s_chtml10_start_br_tag(void *pdoc, Node *node)
     }
   }
   W_L(">");
-  W_NLCODE();
+
   return chtml10->out;
 }
 
@@ -1754,7 +1740,6 @@ s_chtml10_end_tr_tag(void *pdoc, Node *UNUSED(child))
   r       = doc->r;
 
   W_L("<br>");
-  W_NLCODE();
 
   return chtml10->out;
 }
@@ -1872,7 +1857,6 @@ s_chtml10_start_form_tag(void *pdoc, Node *node)
   }
 
   W_L(">");
-  W_NLCODE();
 
   return chtml10->out;
 }
@@ -1898,7 +1882,6 @@ s_chtml10_end_form_tag(void *pdoc, Node *UNUSED(child))
   r       = doc->r;
 
   W_L("</form>");
-  W_NLCODE();
 
   return chtml10->out;
 }
@@ -1984,7 +1967,7 @@ s_chtml10_start_input_tag(void *pdoc, Node *node)
 
   if (value && *value != 0) {
     W_L(" value=\"");
-    W_V(value);
+    W_V(chxj_add_slash_to_doublequote(doc->pool, value));
     W_L("\"");
   }
 
@@ -2014,7 +1997,6 @@ s_chtml10_start_input_tag(void *pdoc, Node *node)
   }
 
   W_L(">");
-  W_NLCODE();
   return chtml10->out;
 }
 
@@ -2081,7 +2063,6 @@ s_chtml10_end_center_tag(void *pdoc, Node *UNUSED(child))
   r       = doc->r;
 
   W_L("</center>");
-  W_NLCODE();
 
   return chtml10->out;
 }
@@ -2182,7 +2163,6 @@ s_chtml10_start_hr_tag(void *pdoc, Node *node)
     }
   }
   W_L(">");
-  W_NLCODE();
   return chtml10->out;
 }
 
@@ -2375,7 +2355,6 @@ s_chtml10_start_img_tag(void *pdoc, Node *node)
   }
 
   W_L(">");
-  W_NLCODE();
   return chtml10->out;
 }
 
@@ -2478,7 +2457,6 @@ s_chtml10_start_select_tag(void *pdoc, Node *child)
   }
 
   W_L(">");
-  W_NLCODE();
   return chtml10->out;
 }
 
@@ -2503,7 +2481,6 @@ s_chtml10_end_select_tag(void *pdoc, Node *UNUSED(child))
   r       = doc->r;
 
   W_L("</select>");
-  W_NLCODE();
   return chtml10->out;
 }
 
@@ -2593,10 +2570,8 @@ static char *
 s_chtml10_end_option_tag(void *pdoc, Node *UNUSED(child))
 {
   chtml10_t *chtml10 = GET_CHTML10(pdoc);
-  Doc       *doc     = chtml10->doc;
 
   /* Don't close */
-  W_NLCODE();
 
   return chtml10->out;
 }
@@ -2648,7 +2623,6 @@ s_chtml10_start_div_tag(void *pdoc, Node *child)
   }
 
   W_L(">");
-  W_NLCODE();
   return chtml10->out;
 }
 
@@ -2673,7 +2647,6 @@ s_chtml10_end_div_tag(void *pdoc, Node *UNUSED(child))
   r       = doc->r;
 
   W_L("</div>");
-  W_NLCODE();
   return chtml10->out;
 }
 
@@ -2730,7 +2703,6 @@ s_chtml10_start_pre_tag(void *pdoc, Node *UNUSED(node))
 
   chtml10->pre_flag++;
   W_L("<pre>");
-  W_NLCODE();
   return chtml10->out;
 }
 
@@ -2755,7 +2727,6 @@ s_chtml10_end_pre_tag(void *pdoc, Node *UNUSED(child))
   r       = doc->r;
 
   W_L("</pre>");
-  W_NLCODE();
   chtml10->pre_flag--;
 
   return chtml10->out;
@@ -2806,7 +2777,6 @@ s_chtml10_start_p_tag(void *pdoc, Node *node)
   }
 
   W_L(">");
-  W_NLCODE();
   return chtml10->out;
 }
 
@@ -2831,7 +2801,6 @@ s_chtml10_end_p_tag(void *pdoc, Node *UNUSED(child))
   r       = doc->r;
 
   W_L("</p>");
-  W_NLCODE();
   return chtml10->out;
 }
 
@@ -2908,7 +2877,6 @@ s_chtml10_start_textarea_tag(void *pdoc, Node *node)
     }
   }
   W_L(">");
-  W_NLCODE();
   return chtml10->out;
 }
 
@@ -2933,7 +2901,6 @@ s_chtml10_end_textarea_tag(void *pdoc, Node *UNUSED(child))
   r       = doc->r;
 
   W_L("</textarea>");
-  W_NLCODE();
   chtml10->textarea_flag--;
 
   return chtml10->out;
@@ -3025,7 +2992,6 @@ s_chtml10_start_blockquote_tag(void *pdoc, Node *UNUSED(child))
   chtml10 = GET_CHTML10(pdoc);
   doc     = chtml10->doc;
   W_L("<blockquote>");
-  W_NLCODE();
   return chtml10->out;
 }
 
@@ -3047,7 +3013,6 @@ s_chtml10_end_blockquote_tag(void *pdoc, Node *UNUSED(child))
   chtml10 = GET_CHTML10(pdoc);
   doc     = chtml10->doc;
   W_L("</blockquote>");
-  W_NLCODE();
   return chtml10->out;
 }
 
@@ -3068,7 +3033,6 @@ s_chtml10_start_dir_tag(void *pdoc, Node *UNUSED(child))
   chtml10 = GET_CHTML10(pdoc);
   doc     = chtml10->doc;
   W_L("<dir>");
-  W_NLCODE();
   return chtml10->out;
 }
 
@@ -3090,7 +3054,6 @@ s_chtml10_end_dir_tag(void *pdoc, Node *UNUSED(child))
   chtml10 = GET_CHTML10(pdoc);
   doc     = chtml10->doc;
   W_L("</dir>");
-  W_NLCODE();
   return chtml10->out;
 }
 
@@ -3131,7 +3094,6 @@ s_chtml10_end_dl_tag(void *pdoc, Node *UNUSED(child))
   chtml10 = GET_CHTML10(pdoc);
   doc     = chtml10->doc;
   W_L("</dl>");
-  W_NLCODE();
   return chtml10->out;
 }
 
@@ -3168,9 +3130,7 @@ static char *
 s_chtml10_end_dt_tag(void *pdoc, Node *UNUSED(child))
 {
   chtml10_t *chtml10 = GET_CHTML10(pdoc);
-  Doc       *doc     = chtml10->doc;
   
-  W_NLCODE();
   return chtml10->out;
 }
 
@@ -3207,8 +3167,6 @@ static char *
 s_chtml10_end_dd_tag(void *pdoc, Node *UNUSED(child))
 {
   chtml10_t *chtml10 = GET_CHTML10(pdoc);
-  Doc       *doc     = chtml10->doc;
-  W_NLCODE();
   return chtml10->out;
 }
 
@@ -3229,7 +3187,6 @@ s_chtml10_start_menu_tag(void *pdoc, Node *UNUSED(child))
   chtml10 = GET_CHTML10(pdoc);
   doc     = chtml10->doc;
   W_L("<menu>");
-  W_NLCODE();
   return chtml10->out;
 }
 
@@ -3248,13 +3205,12 @@ s_chtml10_end_menu_tag(void *pdoc, Node *UNUSED(child))
   chtml10_t *chtml10 = GET_CHTML10(pdoc);
   Doc *doc = chtml10->doc;
   W_L("</menu>");
-  W_NLCODE();
   return chtml10->out;
 }
 
 
 /**
- * It is a hanplaintexter who processes the PLAINTEXT tag.
+ * It is a handler who processes the PLAINTEXT tag.
  *
  * @param pdoc  [i/o] The pointer to the CHTML structure at the output
  *                     destination is specified.
@@ -3293,7 +3249,7 @@ s_chtml10_start_plaintext_tag_inner(void *pdoc, Node *node)
 
 
 /**
- * It is a hanplaintexter who processes the PLAINTEXT tag.
+ * It is a handler who processes the PLAINTEXT tag.
  *
  * @param pdoc  [i/o] The pointer to the CHTML structure at the output
  *                     destination is specified.
@@ -3306,6 +3262,19 @@ s_chtml10_end_plaintext_tag(void *pdoc, Node *UNUSED(child))
   chtml10_t *chtml10 = GET_CHTML10(pdoc);
   return chtml10->out;
 }
+
+
+/**
+ * It is handler who processes the New Line Code.
+ */
+static char *
+s_chtml10_newline_mark(void *pdoc, Node *UNUSED(node))
+{
+  chtml10_t *chtml10 = GET_CHTML10(pdoc);
+  Doc *doc = chtml10->doc;
+  W_NLCODE();
+  return chtml10->out;
+}
 /*
  * vim:ts=2 et
  */
