@@ -113,6 +113,7 @@ static char *s_chtml50_end_menu_tag       (void *pdoc, Node *node);
 static char *s_chtml50_start_plaintext_tag       (void *pdoc, Node *node);
 static char *s_chtml50_start_plaintext_tag_inner (void *pdoc, Node *node);
 static char *s_chtml50_end_plaintext_tag         (void *pdoc, Node *node);
+static char *s_chtml50_newline_mark       (void *pdoc, Node *node);
 
 static void  s_init_chtml50(chtml50_t *chtml, Doc *doc, request_rec *r, device_table *spec);
 
@@ -385,6 +386,11 @@ tag_handler chtml50_handler[] = {
     s_chtml50_start_marquee_tag,
     s_chtml50_end_marquee_tag,
   },
+  /* tagNLMARK */
+  {
+    s_chtml50_newline_mark,
+    NULL,
+  },
 };
 
 
@@ -581,7 +587,6 @@ s_chtml50_start_html_tag(void *pdoc, Node *UNUSED(node))
   /* start HTML tag                                                           */
   /*--------------------------------------------------------------------------*/
   W_L("<html>");
-  W_NLCODE();
 
   return chtml50->out;
 }
@@ -602,7 +607,6 @@ s_chtml50_end_html_tag(void *pdoc, Node *UNUSED(child))
   Doc         *doc     = chtml50->doc;
 
   W_L("</html>");
-  W_NLCODE();
 
   return chtml50->out;
 }
@@ -705,7 +709,6 @@ s_chtml50_start_meta_tag(void *pdoc, Node *node)
     }
   }
   W_L(">");
-  W_NLCODE();
 
   return chtml50->out;
 }
@@ -743,7 +746,6 @@ s_chtml50_start_head_tag(void* pdoc, Node* UNUSED(node))
   Doc         *doc     = chtml50->doc;
 
   W_L("<head>");
-  W_NLCODE();
 
   return chtml50->out;
 }
@@ -764,7 +766,6 @@ s_chtml50_end_head_tag(void *pdoc, Node *UNUSED(node))
   Doc       *doc     = chtml50->doc;
 
   W_L("</head>");
-  W_NLCODE();
 
   return chtml50->out;
 }
@@ -805,7 +806,6 @@ s_chtml50_end_title_tag(void *pdoc, Node *UNUSED(child))
   Doc          *doc     = chtml50->doc;
 
   W_L("</title>");
-  W_NLCODE();
 
   return chtml50->out;
 }
@@ -847,7 +847,6 @@ s_chtml50_start_base_tag(void *pdoc, Node *node)
     }
   }
   W_L(">");
-  W_NLCODE();
 
   return chtml50->out;
 }
@@ -941,7 +940,6 @@ s_chtml50_start_body_tag(void *pdoc, Node *node)
     }
   }
   W_L(">");
-  W_NLCODE();
 
   return chtml50->out;
 }
@@ -965,7 +963,6 @@ s_chtml50_end_body_tag(void *pdoc, Node *UNUSED(child))
   doc     = chtml50->doc;
 
   W_L("</body>");
-  W_NLCODE();
 
   return chtml50->out;
 }
@@ -1113,7 +1110,6 @@ s_chtml50_end_a_tag(void *pdoc, Node *UNUSED(child))
   doc     = chtml50->doc;
 
   W_L("</a>");
-  W_NLCODE();
 
   return chtml50->out;
 }
@@ -1156,7 +1152,6 @@ s_chtml50_start_br_tag(void *pdoc, Node *node)
     }
   }
   W_L(">");
-  W_NLCODE();
   return chtml50->out;
 }
 
@@ -1215,7 +1210,6 @@ s_chtml50_end_tr_tag(void *pdoc, Node *UNUSED(child))
   r       = doc->r;
 
   W_L("<br>");
-  W_NLCODE();
 
   return chtml50->out;
 }
@@ -1332,7 +1326,6 @@ s_chtml50_end_font_tag(void *pdoc, Node *UNUSED(child))
 
   if (chtml50->font_flag) {
     W_L("</font>");
-    W_NLCODE();
     chtml50->font_flag--;
   }
 
@@ -1396,7 +1389,6 @@ s_chtml50_start_form_tag(void *pdoc, Node *node)
     }
   }
   W_L(">");
-  W_NLCODE();
   return chtml50->out;
 }
 
@@ -1419,7 +1411,6 @@ s_chtml50_end_form_tag(void *pdoc, Node *UNUSED(child))
   doc     = chtml50->doc;
 
   W_L("</form>");
-  W_NLCODE();
 
   return chtml50->out;
 }
@@ -1490,7 +1481,7 @@ s_chtml50_start_input_tag(void *pdoc, Node *node)
   }
   if (value && *value) {
     W_L(" value=\"");
-    W_V(value);
+    W_V(chxj_add_slash_to_doublequote(doc->pool, value));
     W_L("\"");
   }
   if (accesskey && *accesskey) {
@@ -1529,7 +1520,6 @@ s_chtml50_start_input_tag(void *pdoc, Node *node)
     W_L(" checked");
   }
   W_L(">");
-  W_NLCODE();
 
   return chtml50->out;
 }
@@ -1570,7 +1560,6 @@ s_chtml50_start_center_tag(void *pdoc, Node *UNUSED(node))
   doc     = chtml50->doc;
 
   W_L("<center>");
-  W_NLCODE();
 
   return chtml50->out;
 }
@@ -1594,7 +1583,6 @@ s_chtml50_end_center_tag(void *pdoc, Node *UNUSED(child))
   doc     = chtml50->doc;
 
   W_L("</center>");
-  W_NLCODE();
 
   return chtml50->out;
 }
@@ -1673,7 +1661,6 @@ s_chtml50_start_hr_tag(void *pdoc, Node *node)
     }
   }
   W_L(">");
-  W_NLCODE();
 
   return chtml50->out;
 }
@@ -1824,7 +1811,6 @@ s_chtml50_start_img_tag(void *pdoc, Node *node)
     }
   }
   W_L(">");
-  W_NLCODE();
 
   return chtml50->out;
 }
@@ -1904,7 +1890,6 @@ s_chtml50_start_select_tag(void *pdoc, Node *child)
     W_L(" multiple");
   }
   W_L(">");
-  W_NLCODE();
 
   return chtml50->out;
 }
@@ -1925,7 +1910,6 @@ s_chtml50_end_select_tag(void *pdoc, Node *UNUSED(child))
   Doc          *doc   = chtml50->doc;
 
   W_L("</select>");
-  W_NLCODE();
 
   return chtml50->out;
 }
@@ -2049,7 +2033,6 @@ s_chtml50_start_div_tag(void *pdoc, Node *child)
     W_L("\"");
   }
   W_L(">");
-  W_NLCODE();
 
   return chtml50->out;
 }
@@ -2070,7 +2053,6 @@ s_chtml50_end_div_tag(void *pdoc, Node *UNUSED(node))
   Doc          *doc     = chtml50->doc;
 
   W_L("</div>");
-  W_NLCODE();
 
   return chtml50->out;
 }
@@ -2108,7 +2090,6 @@ s_chtml50_start_ul_tag(void *pdoc, Node *node)
     }
   }
   W_L(">");
-  W_NLCODE();
   return chtml50->out;
 }
 
@@ -2128,7 +2109,6 @@ s_chtml50_end_ul_tag(void *pdoc, Node *UNUSED(child))
   Doc           *doc     = chtml50->doc;
 
   W_L("</ul>");
-  W_NLCODE();
 
   return chtml50->out;
 }
@@ -2150,7 +2130,6 @@ s_chtml50_start_pre_tag(void *pdoc, Node *UNUSED(node))
 
   chtml50->pre_flag++;
   W_L("<pre>");
-  W_NLCODE();
 
   return chtml50->out;
 }
@@ -2171,7 +2150,6 @@ s_chtml50_end_pre_tag(void *pdoc, Node *UNUSED(child))
   Doc           *doc     = chtml50->doc;
 
   W_L("</pre>");
-  W_NLCODE();
   chtml50->pre_flag--;
 
   return chtml50->out;
@@ -2221,7 +2199,6 @@ s_chtml50_start_p_tag(void *pdoc, Node *node)
     W_L("\"");
   }
   W_L(">");
-  W_NLCODE();
   return chtml50->out;
 }
 
@@ -2244,7 +2221,6 @@ s_chtml50_end_p_tag(void *pdoc, Node *UNUSED(child))
   doc     = chtml50->doc;
 
   W_L("</p>");
-  W_NLCODE();
 
   return chtml50->out;
 }
@@ -2291,7 +2267,6 @@ s_chtml50_start_ol_tag(void *pdoc, Node *node)
     }
   }
   W_L(">");
-  W_NLCODE();
 
   return chtml50->out;
 }
@@ -2312,7 +2287,6 @@ s_chtml50_end_ol_tag(void *pdoc, Node *UNUSED(node))
   Doc       *doc     = chtml50->doc;
 
   W_L("</ol>");
-  W_NLCODE();
 
   return chtml50->out;
 }
@@ -2436,7 +2410,6 @@ s_chtml50_end_h1_tag(void *pdoc, Node *UNUSED(child))
   doc     = chtml50->doc;
 
   W_L("</h1>");
-  W_NLCODE();
 
   return chtml50->out;
 }
@@ -2497,7 +2470,6 @@ s_chtml50_end_h2_tag(void *pdoc, Node *UNUSED(child))
   Doc         *doc     = chtml50->doc;
 
   W_L("</h2>");
-  W_NLCODE();
 
   return chtml50->out;
 }
@@ -2559,7 +2531,6 @@ s_chtml50_end_h3_tag(void *pdoc, Node *UNUSED(child))
   Doc       *doc   = chtml50->doc;
 
   W_L("</h3>");
-  W_NLCODE();
 
   return chtml50->out;
 }
@@ -2620,7 +2591,6 @@ s_chtml50_end_h4_tag(void *pdoc, Node *UNUSED(child))
   Doc       *doc     = chtml50->doc;
 
   W_L("</h4>");
-  W_NLCODE();
 
   return chtml50->out;
 }
@@ -2681,7 +2651,6 @@ s_chtml50_end_h5_tag(void *pdoc, Node *UNUSED(child))
   Doc       *doc     = chtml50->doc;
 
   W_L("</h5>");
-  W_NLCODE();
 
   return chtml50->out;
 }
@@ -2742,7 +2711,6 @@ s_chtml50_end_h6_tag(void *pdoc, Node *UNUSED(child))
   Doc       *doc     = chtml50->doc;
 
   W_L("</h6>");
-  W_NLCODE();
 
   return chtml50->out;
 }
@@ -2803,7 +2771,6 @@ s_chtml50_start_textarea_tag(void *pdoc, Node *node)
     }
   }
   W_L(">");
-  W_NLCODE();
   return chtml50->out;
 }
 
@@ -2823,7 +2790,6 @@ s_chtml50_end_textarea_tag(void *pdoc, Node *UNUSED(child))
   Doc       *doc     = chtml50->doc;
 
   W_L("</textarea>");
-  W_NLCODE();
   chtml50->textarea_flag--;
 
   return chtml50->out;
@@ -2931,7 +2897,6 @@ s_chtml50_start_blockquote_tag(void *pdoc, Node *UNUSED(child))
   chtml50_t *chtml50 = GET_CHTML50(pdoc);
   Doc       *doc     = chtml50->doc;
   W_L("<blockquote>");
-  W_NLCODE();
   return chtml50->out;
 }
 
@@ -2950,7 +2915,6 @@ s_chtml50_end_blockquote_tag(void *pdoc, Node *UNUSED(child))
   chtml50_t *chtml50 = GET_CHTML50(pdoc);
   Doc       *doc     = chtml50->doc;
   W_L("</blockquote>");
-  W_NLCODE();
   return chtml50->out;
 }
 
@@ -2984,7 +2948,6 @@ s_chtml50_start_dir_tag(void *pdoc, Node *node)
     }
   }
   W_L(">");
-  W_NLCODE();
   return chtml50->out;
 }
 
@@ -3003,7 +2966,6 @@ s_chtml50_end_dir_tag(void *pdoc, Node *UNUSED(child))
   chtml50_t *chtml50 = GET_CHTML50(pdoc);
   Doc       *doc = chtml50->doc;
   W_L("</dir>");
-  W_NLCODE();
   return chtml50->out;
 }
 
@@ -3022,7 +2984,6 @@ s_chtml50_start_dl_tag(void *pdoc, Node *UNUSED(child))
   chtml50_t *chtml50 = GET_CHTML50(pdoc);
   Doc       *doc     = chtml50->doc;
   W_L("<dl>");
-  W_NLCODE();
   return chtml50->out;
 }
 
@@ -3041,7 +3002,6 @@ s_chtml50_end_dl_tag(void *pdoc, Node *UNUSED(child))
   chtml50_t *chtml50 = GET_CHTML50(pdoc);
   Doc       *doc     = chtml50->doc;
   W_L("</dl>");
-  W_NLCODE();
   return chtml50->out;
 }
 
@@ -3189,7 +3149,6 @@ s_chtml50_end_marquee_tag(void *pdoc, Node *UNUSED(child))
   chtml50_t *chtml50 = GET_CHTML50(pdoc);
   Doc       *doc = chtml50->doc;
   W_L("</marquee>");
-  W_NLCODE();
   return chtml50->out;
 }
 
@@ -3226,7 +3185,6 @@ s_chtml50_end_blink_tag(void *pdoc, Node *UNUSED(child))
   chtml50_t *chtml50 = GET_CHTML50(pdoc);
   Doc       *doc = chtml50->doc;
   W_L("</blink>");
-  W_NLCODE();
   return chtml50->out;
 }
 
@@ -3263,7 +3221,6 @@ s_chtml50_start_menu_tag(void *pdoc, Node *node)
     }
   }
   W_L(">");
-  W_NLCODE();
   return chtml50->out;
 }
 
@@ -3282,7 +3239,6 @@ s_chtml50_end_menu_tag(void *pdoc, Node *UNUSED(child))
   chtml50_t *chtml50 = GET_CHTML50(pdoc);
   Doc       *doc = chtml50->doc;
   W_L("</menu>");
-  W_NLCODE();
   return chtml50->out;
 }
 
@@ -3335,6 +3291,19 @@ s_chtml50_end_plaintext_tag(void *pdoc, Node *UNUSED(child))
   chtml50_t *chtml50 = GET_CHTML50(pdoc);
   return chtml50->out;
 }
+
+
+/**
+ * It is handler who processes the New Line Code.
+ */
+static char *
+s_chtml50_newline_mark(void *pdoc, Node *UNUSED(node))
+{
+  chtml50_t *chtml50 = GET_CHTML50(pdoc);
+  Doc *doc = chtml50->doc;
+  W_NLCODE();
+  return chtml50->out;
+}
 /*
  * vim:ts=2 et
  */
